// Copyright David Abrahams 2006. Distributed under the Boost
// Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
#ifndef BOOST_DETAIL_FUNCTION3_DWA2006514_HPP
# define BOOST_DETAIL_FUNCTION3_DWA2006514_HPP

# define args (3)
# include <boost/accumulators/numeric/detail/function_n.hpp>

#endif // BOOST_DETAIL_FUNCTION3_DWA2006514_HPP
