// Copyright (c) 2022 Denis Mikhailov
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_PFR_DETAIL_TRAITS_FWD_HPP
#define BOOST_PFR_DETAIL_TRAITS_FWD_HPP
#pragma once

#include <boost/pfr/detail/config.hpp>

namespace boost { namespace pfr {

template<class T, class WhatFor>
struct is_reflectable;

}} // namespace boost::pfr

#endif // BOOST_PFR_DETAIL_TRAITS_FWD_HPP


